/**
 * @file artis/kernel/Externals.hpp
 * @author See the AUTHORS file
 */

/*
 * Copyright (C) 2012-2017 ULCO http://www.univ-littoral.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ARTIS_KERNEL_EXTERNALS_HPP
#define __ARTIS_KERNEL_EXTERNALS_HPP

#include <artis/kernel/Macro.hpp>
#include <artis/kernel/State.hpp>
#include <vector>

namespace artis { namespace kernel {

template < typename T, typename U >
class Externals
{
    template < typename V >
    struct element
    {
        unsigned int index;
        const std::string name;
        V T::* var;

        element(unsigned int index, const std::string& name, V T::* var) :
            index(index), name(name), var(var)
        { }
    };

public:
    Externals() : updated(false)
    { }

    virtual ~Externals()
    { }

    bool check(typename U::type t) const
    {
        bool OK = true;
        typename std::vector <
            std::pair < double,
                        Any > >::const_iterator it = externals.begin();

        while (it != externals.end() and OK) {
            OK = it->first == t;
            ++it;
        }
        return OK;
    }

    template < typename V >
    void E_(std::initializer_list < element < V > > list)
    {
        for (typename std::initializer_list < element < V > >::iterator it =
                 list.begin(); it != list.end(); ++it) {
            if (externals.size() <= it->index) {
                externals.resize(it->index + 1, std::make_pair(-1, Any()));
                external_names.resize(it->index + 1, std::string());
            }
            externals[it->index] = std::make_pair(-1, it->var);
            external_names[it->index] = it->name;
        }
    }

    template < typename V >
    void external_(unsigned int index, const std::string& name, V T::* var)
    {
        if (externals.size() <= index) {
            externals.resize(index + 1, std::make_pair(-1, Any()));
            external_names.resize(index + 1, std::string());
        }
        externals[index] = std::make_pair(-1, var);
        external_names[index] = name;
    }

    bool is_ready(typename U::type t, unsigned int index) const
    { return externals.at(index).first == t; }

    template < typename V >
    void put(typename U::type t, unsigned int index, V value)
    {
        if (externals.at(index).first != t) {
            Any v = externals.at(index).second;

            v.put < T, V >(static_cast < T* >(this), value);
            externals.at(index).first = t;
            updated = true;
        } else {
            Any v = externals.at(index).second;

            if (static_cast < const T* >(this)->*(v.get < T, V >()) != value) {
                v.put < T, V >(static_cast < T* >(this), value);
                updated = true;
            }
        }
    }

    unsigned int size() const
    { return externals.size(); }

protected:
    bool updated;
    std::vector < std::pair < typename U::type, Any > > externals;
    std::vector < std::string > external_names;
};

#define External(index, var)                                    \
        external_(index, std::string(ESCAPEQUOTE(index)), var)

#define Externals(V,L) E_< V >(UNWRAP2 L)

} } // namespace artis kernel

#endif
