/**
 * @file artis/kernel/Internals.hpp
 * @author See the AUTHORS file
 */

/*
 * Copyright (C) 2012-2017 ULCO http://www.univ-littoral.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ARTIS_KERNEL_INTERNALS_HPP
#define __ARTIS_KERNEL_INTERNALS_HPP

#include <artis/kernel/Macro.hpp>
#include <artis/kernel/State.hpp>
#include <artis/kernel/Value.hpp>

#include <vector>

namespace artis { namespace kernel {

template < typename T, typename U >
class Internals
{
    template < typename V >
    struct element
    {
        unsigned int index;
        const std::string name;
        V T::* var;

        element(unsigned int index, const std::string& name, V T::* var) :
            index(index), name(name), var(var)
        { }
    };

public:
    Internals()
    { }

    virtual ~Internals()
    { }

    const Any& get(unsigned int index) const
    { return internals.at(index); }

    template < typename V >
    void I_(std::initializer_list < element < V > > list)
    {
        for (typename std::initializer_list <element < V > >::iterator it =
                 list.begin(); it != list.end(); ++it) {
            if (internals.size() <= it->index) {
                internals.resize(it->index + 1);
                internal_names.resize(it->index + 1, std::string());
            }
            internals[it->index] = it->var;
            internal_names[it->index] = it->name;
        }
    }

    template < typename V >
    void internal_(unsigned int index, const std::string& name, V T::* var)
    {
        if (internals.size() <= index) {
            internals.resize(index + 1, Any());
            internal_names.resize(index + 1, std::string());
        }
        internals[index] = Any(var);
        internal_names[index] = name;
    }

    const std::string& name(unsigned int index) const
    { return internal_names.at(index); }

    virtual void restore(const State < U >& /* state */)
    {
        // unsigned int index = 0;

        // for (typename std::vector < Any >::iterator it =
        //          internals.begin(); it != internals.end(); ++it) {
        //     state.get_internal(index, *it);
        //     ++index;
        // }
        // index = 0;
        // for (typename std::vector < int T::* >::iterator it =
        //          internalsI.begin(); it != internalsI.end(); ++it) {
        //     state.get_internal(index, *it);
        //     ++index;
        // }
        // index = 0;
        // for (typename std::vector < double T::* >::iterator it =
        //          internalsD.begin(); it != internalsD.end(); ++it) {
        //     state.get_internal(index, *it);
        //     ++index;
        // }
    }

    virtual void save(State < U >& /* state */) const
    {
        // unsigned int index = 0;

        // for (typename std::vector < bool T::* >::const_iterator it =
        //          internalsB.begin(); it != internalsB.end(); ++it) {
        //     state.add_internal(index, *it);
        //     ++index;
        // }
        // index = 0;
        // for (typename std::vector < int T::* >::const_iterator it =
        //          internalsI.begin(); it != internalsI.end(); ++it) {
        //     state.add_internal(index, *it);
        //     ++index;
        // }
        // index = 0;
        // for (typename std::vector < double T::* >::const_iterator it =
        //          internalsD.begin(); it != internalsD.end(); ++it) {
        //     state.add_internal(index, *it);
        //     ++index;
        // }
    }

    unsigned int size() const
    { return internals.size(); }

private:
    std::vector < Any > internals;
    std::vector < std::string > internal_names;
};

#define Internal(index, var)                                    \
        internal_(index, std::string(ESCAPEQUOTE(index)), var)

#define Internals(V,L) I_< V >(UNWRAP2 L)

} } // namespace artis kernel

#endif
