/**
 * @file artis/kernel/Values.hpp
 * @author See the AUTHORS file
 */

/*
 * Copyright (C) 2012-2017 ULCO http://www.univ-littoral.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ARTIS_KERNEL_VALUES_HPP
#define __ARTIS_KERNEL_VALUES_HPP

#include <artis/kernel/Value.hpp>

#include <map>

#include <boost/serialization/serialization.hpp>
#include <boost/serialization/map.hpp>

namespace artis { namespace kernel {

class Values
{
public:
    Values()
    { }

    virtual ~Values()
    { }

    template < typename T >
    void add_internal(unsigned int key, T value)
    { _internals[key] = Value(value); }

    template < typename T >
    void add_state(unsigned int key, T value)
    { _states[key] = Value(value); }

    template < typename T >
    void get_internal(unsigned int key, T& value) const
    {
        std::map < unsigned int, Value >::const_iterator it =
            _internals.find(key);

        if (it != _internals.end()) {
            it->second.get_content(value);
        } else {
            assert(false);
        }
    }

    template < typename T >
    void get_state(unsigned int key, T& value) const
    {
        std::map < unsigned int, Value >::const_iterator it =
            _states.find(key);

        if (it != _states.end()) {
            it->second.get_content(value);
        } else {
            assert(false);
        }
    }

private:
    friend class boost::serialization::access;

    template < class Archive >
    void serialize(Archive & ar, const unsigned int version)
    {
        (void) version;

        ar & _internals;
        ar & _states;
    }

    std::map < unsigned int, Value > _internals;
    std::map < unsigned int, Value > _states;
};

} }

#endif
